@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,18 +16,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
-#include <hydrogen/config.h>
-#include <hydrogen/version.h>
-#include <hydrogen/hydrogen.h>
-#include <hydrogen/event_queue.h>
-#include <hydrogen/fx/LadspaFX.h>
-#include <hydrogen/Preferences.h>
-#include <hydrogen/helpers/filesystem.h>
+#include <core/config.h>
+#include <core/Version.h>
+#include <core/Hydrogen.h>
+#include <core/EventQueue.h>
+#include <core/FX/LadspaFX.h>
+#include <core/Preferences.h>
+#include <core/Helpers/Filesystem.h>
 
 #include "HydrogenApp.h"
 #include "Skin.h"
@@ -34,7 +34,7 @@
 #include "MainForm.h"
 #include "PlayerControl.h"
 #include "AudioEngineInfoForm.h"
-#include "HelpBrowser.h"
+#include "FilesystemInfoForm.h"
 #include "LadspaFXProperties.h"
 #include "InstrumentRack.h"
 #include "Director.h"
@@ -51,13 +51,14 @@
 #include "Mixer/MixerLine.h"
 #include "UndoActions.h"
 
+#include <core/Basics/PatternList.h>
+#include <core/Basics/InstrumentList.h>
+
 #include "Widgets/InfoBar.h"
 
-
 #include <QtGui>
-#if QT_VERSION >= 0x050000
-#  include <QtWidgets>
-#endif
+#include <QtWidgets>
+
 
 using namespace H2Core;
 
@@ -72,8 +73,6 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
  , m_pPatternEditorPanel( nullptr )
  , m_pAudioEngineInfoForm( nullptr )
  , m_pSongEditorPanel( nullptr )
- , m_pHelpBrowser( nullptr )
- , m_pFirstTimeInfo( nullptr )
  , m_pPlayerControl( nullptr )
  , m_pPlaylistDialog( nullptr )
  , m_pSampleEditor( nullptr )
@@ -86,11 +85,12 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
 	connect( m_pEventQueueTimer, SIGNAL( timeout() ), this, SLOT( onEventQueueTimer() ) );
 	m_pEventQueueTimer->start( QUEUE_TIMER_PERIOD );
 
-
-	// Create the audio engine :)
-	Hydrogen::create_instance();
-	Hydrogen::get_instance()->setSong( pFirstSong );
-	Preferences::get_instance()->setLastSongFilename( pFirstSong->get_filename() );
+	if ( ! Hydrogen::get_instance()->isUnderSessionManagement() ) {
+		// When under Non Session Management the new Song will be
+		// loaded by the corresponding NSM client instance.
+		Hydrogen::get_instance()->setSong( pFirstSong );
+	} 
+	
 	SoundLibraryDatabase::create_instance();
 
 	//setup the undo stack
@@ -105,16 +105,15 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
 	// restore audio engine form properties
 	m_pAudioEngineInfoForm = new AudioEngineInfoForm( nullptr );
 	WindowProperties audioEngineInfoProp = pPref->getAudioEngineInfoProperties();
-	m_pAudioEngineInfoForm->move( audioEngineInfoProp.x, audioEngineInfoProp.y );
-	if ( audioEngineInfoProp.visible ) {
-		m_pAudioEngineInfoForm->show();
-	}
-	else {
-		m_pAudioEngineInfoForm->hide();
-	}
+	setWindowProperties( m_pAudioEngineInfoForm, audioEngineInfoProp, SetX + SetY );
+	
+	m_pFilesystemInfoForm = new FilesystemInfoForm( nullptr );
 
 	m_pPlaylistDialog = new PlaylistDialog( nullptr );
 	m_pDirector = new Director( nullptr );
+
+	// Initially keyboard cursor is hidden.
+	m_bHideKeyboardCursor = true;
 	
 	// Since HydrogenApp does implement some handler functions for
 	// Events as well, it should be registered as an Eventlistener
@@ -122,6 +121,66 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm, Song *pFirstSong )
 	addEventListener( this );
 }
 
+void HydrogenApp::setWindowProperties( QWidget *pWindow, WindowProperties &prop, unsigned flags ) {
+	if ( flags & SetVisible ) {
+		if ( prop.visible) {
+			pWindow->show();
+		} else {
+			pWindow->hide();
+		}
+	}
+
+	// Preserve the current values of anything we're not setting.
+	QRect oldGeometry = pWindow->geometry();
+	if ( prop.m_geometry.size() == 0 ) {
+		// No geometry saved in preferences. Most likely an old preferences file. Set the size and shape
+		pWindow->resize( prop.width, prop.height );
+		pWindow->move( prop.x, prop.y );
+		prop.m_geometry = pWindow->saveGeometry();
+	}
+
+	// restore geometry will also ensure things are visible if screen geometry has changed.
+	pWindow->restoreGeometry( prop.m_geometry );
+
+	// For anything that we're not setting, restore the previous values.
+	QRect newGeometry = pWindow->geometry();
+	if ( !( flags & SetX ) ) {
+		newGeometry.setX( oldGeometry.x() );
+	}
+	if ( !( flags & SetY ) ) {
+		newGeometry.setY( oldGeometry.y() );
+	}
+	if ( !( flags & SetWidth ) ) {
+		newGeometry.setWidth( oldGeometry.width() );
+	}
+	if ( !( flags & SetHeight ) ) {
+		newGeometry.setHeight( oldGeometry.height() );
+	}
+
+	// If a window is fixed-size, don't restore it full-screen (macOS sometimes does this, annoyingly)
+	if ( pWindow->minimumSize() == pWindow->maximumSize() ) {
+		if ( pWindow->isFullScreen()) {
+			pWindow->showNormal();
+		}
+	}
+
+	if ( oldGeometry != newGeometry ) {
+		pWindow->setGeometry( newGeometry );
+	}
+}
+
+
+WindowProperties HydrogenApp::getWindowProperties( QWidget *pWindow ) {
+	WindowProperties prop;
+	prop.x = pWindow->x();
+	prop.y = pWindow->y();
+	prop.height = pWindow->height();
+	prop.width = pWindow->width();
+	prop.visible = pWindow->isVisible();
+	prop.m_geometry = pWindow->saveGeometry();
+
+	return prop;
+}
 
 
 HydrogenApp::~HydrogenApp()
@@ -133,8 +192,8 @@ HydrogenApp::~HydrogenApp()
 	//delete the undo tmp directory
 	cleanupTemporaryFiles();
 
-	delete m_pHelpBrowser;
 	delete m_pAudioEngineInfoForm;
+	delete m_pFilesystemInfoForm;
 	delete m_pMixer;
 	delete m_pPlaylistDialog;
 	delete m_pDirector;
@@ -178,29 +237,32 @@ void HydrogenApp::setupSinglePanedInterface()
 
 	// MAINFORM
 	WindowProperties mainFormProp = pPref->getMainFormProperties();
-	m_pMainForm->resize( mainFormProp.width, mainFormProp.height );
-	m_pMainForm->move( mainFormProp.x, mainFormProp.y );
+	setWindowProperties( m_pMainForm, mainFormProp, SetDefault & ~SetVisible );
 
 	m_pSplitter = new QSplitter( nullptr );
 	m_pSplitter->setOrientation( Qt::Vertical );
 	m_pSplitter->setOpaqueResize( true );
 
 	m_pTab = new QTabWidget( nullptr );
+	m_pTab->setObjectName( "TabbedInterface" );
 
 	// SONG EDITOR
-	if( uiLayout == Preferences::UI_LAYOUT_SINGLE_PANE)
+	if( uiLayout == Preferences::UI_LAYOUT_SINGLE_PANE) {
 		m_pSongEditorPanel = new SongEditorPanel( m_pSplitter );
-	else
+	} else {
 		m_pSongEditorPanel = new SongEditorPanel( m_pTab );
+	}
 
 	WindowProperties songEditorProp = pPref->getSongEditorProperties();
-	m_pSongEditorPanel->resize( songEditorProp.width, songEditorProp.height );
+	setWindowProperties( m_pSongEditorPanel, songEditorProp, SetWidth + SetHeight );
 
-	if( uiLayout == Preferences::UI_LAYOUT_TABBED)
+	if( uiLayout == Preferences::UI_LAYOUT_TABBED) {
 		m_pTab->addTab( m_pSongEditorPanel, tr("Song Editor") );
+	}
 
 	// this HBox will contain the InstrumentRack and the Pattern editor
 	QWidget *pSouthPanel = new QWidget( m_pSplitter );
+	pSouthPanel->setObjectName( "SouthPanel" );
 	QHBoxLayout *pEditorHBox = new QHBoxLayout();
 	pEditorHBox->setSpacing( 5 );
 	pEditorHBox->setMargin( 0 );
@@ -208,6 +270,8 @@ void HydrogenApp::setupSinglePanedInterface()
 
 	// INSTRUMENT RACK
 	m_pInstrumentRack = new InstrumentRack( nullptr );
+	WindowProperties instrumentRackProp = pPref->getInstrumentRackProperties();
+	m_pInstrumentRack->setHidden( !instrumentRackProp.visible );
 
 	if( uiLayout == Preferences::UI_LAYOUT_TABBED ){
 		m_pTab->setMovable( false );
@@ -218,7 +282,7 @@ void HydrogenApp::setupSinglePanedInterface()
 	// PATTERN EDITOR
 	m_pPatternEditorPanel = new PatternEditorPanel( nullptr );
 	WindowProperties patternEditorProp = pPref->getPatternEditorProperties();
-	m_pPatternEditorPanel->resize( patternEditorProp.width, patternEditorProp.height );
+	setWindowProperties( m_pPatternEditorPanel, patternEditorProp, SetWidth + SetHeight );
 
 	pEditorHBox->addWidget( m_pPatternEditorPanel );
 	pEditorHBox->addWidget( m_pInstrumentRack );
@@ -231,24 +295,21 @@ void HydrogenApp::setupSinglePanedInterface()
 	m_pMainForm->setCentralWidget( mainArea );
 
 	// LAYOUT!!
-	QVBoxLayout *pMainVBox = new QVBoxLayout();
-	pMainVBox->setSpacing( 1 );
-	pMainVBox->setMargin( 0 );
-	pMainVBox->addWidget( m_pPlayerControl );
+	m_pMainVBox = new QVBoxLayout();
+	m_pMainVBox->setSpacing( 1 );
+	m_pMainVBox->setMargin( 0 );
+	m_pMainVBox->addWidget( m_pPlayerControl );
 
-	m_pInfoBar = new InfoBar();
-	m_pInfoBar->hide();
-	pMainVBox->addWidget( m_pInfoBar );
-	pMainVBox->addSpacing( 3 );
+	m_pMainVBox->addSpacing( 3 );
 
-	if( uiLayout == Preferences::UI_LAYOUT_SINGLE_PANE)
-		pMainVBox->addWidget( m_pSplitter );
-	else {
-		pMainVBox->addWidget( m_pTab );
+	if( uiLayout == Preferences::UI_LAYOUT_SINGLE_PANE) {
+		m_pMainVBox->addWidget( m_pSplitter );
+	} else {
+		m_pMainVBox->addWidget( m_pTab );
 
 	}
 
-	mainArea->setLayout( pMainVBox );
+	mainArea->setLayout( m_pMainVBox );
 
 
 
@@ -256,9 +317,10 @@ void HydrogenApp::setupSinglePanedInterface()
 	// MIXER
 	m_pMixer = new Mixer(nullptr);
 	WindowProperties mixerProp = pPref->getMixerProperties();
-
-	m_pMixer->resize( mixerProp.width, mixerProp.height );
-	m_pMixer->move( mixerProp.x, mixerProp.y );
+	if ( uiLayout != Preferences::UI_LAYOUT_SINGLE_PANE ) {
+		mixerProp.visible = false;
+	}
+	setWindowProperties( m_pMixer, mixerProp );
 
 	if( uiLayout == Preferences::UI_LAYOUT_TABBED){
 		m_pTab->addTab(m_pMixer,tr("Mixer"));
@@ -266,32 +328,15 @@ void HydrogenApp::setupSinglePanedInterface()
 
 	m_pMixer->updateMixer();
 
-	if ( mixerProp.visible && uiLayout == Preferences::UI_LAYOUT_SINGLE_PANE ) {
-		m_pMixer->show();
-	}
-	else {
-		m_pMixer->hide();
-	}
 
 
-	// HELP BROWSER
-	QString sDocPath = H2Core::Filesystem::doc_dir();
-	QString sDocURI = sDocPath + "/manual.html";
-	m_pHelpBrowser = new SimpleHTMLBrowser( nullptr, sDocPath, sDocURI, SimpleHTMLBrowser::MANUAL );
-
 #ifdef H2CORE_HAVE_LADSPA
 	// LADSPA FX
 	for (uint nFX = 0; nFX < MAX_FX; nFX++) {
 		m_pLadspaFXProperties[nFX] = new LadspaFXProperties( nullptr, nFX );
 		m_pLadspaFXProperties[nFX]->hide();
 		WindowProperties prop = pPref->getLadspaProperties(nFX);
-		m_pLadspaFXProperties[nFX]->move( prop.x, prop.y );
-		if ( prop.visible ) {
-			m_pLadspaFXProperties[nFX]->show();
-		}
-		else {
-			m_pLadspaFXProperties[nFX]->hide();
-		}
+		setWindowProperties( m_pLadspaFXProperties[ nFX ], prop, SetWidth + SetHeight );
 	}
 #endif
 
@@ -302,6 +347,13 @@ void HydrogenApp::setupSinglePanedInterface()
 }
 
 
+InfoBar *HydrogenApp::addInfoBar() {
+	InfoBar *pInfoBar = new InfoBar();
+	m_pMainVBox->insertWidget( 1, pInfoBar );
+	return pInfoBar;
+}
+
+
 
 void HydrogenApp::currentTabChanged(int index)
 {
@@ -317,17 +369,26 @@ void HydrogenApp::closeFXProperties()
 #endif
 }
 
-void HydrogenApp::setSong(Song* song)
-{
-	Hydrogen::get_instance()->setSong( song );
-	Preferences::get_instance()->setLastSongFilename( song->get_filename() );
+bool HydrogenApp::openSong( const QString sFilename ) {
 
-	m_pSongEditorPanel->updateAll();
-	m_pPatternEditorPanel->updateSLnameLabel();
+	auto pCoreActionController = Hydrogen::get_instance()->getCoreActionController();
+	if ( ! pCoreActionController->openSong( sFilename ) ) {
+		QMessageBox::information( m_pMainForm, "Hydrogen", tr("Error loading song.") );
+		return false;
+	}
 
-	updateWindowTitle();
+	return true;
+}
 
-	m_pMainForm->updateRecentUsedSongList();
+bool HydrogenApp::openSong( H2Core::Song* pSong ) {
+
+	auto pCoreActionController = Hydrogen::get_instance()->getCoreActionController();
+	if ( ! pCoreActionController->openSong( pSong ) ) {
+		QMessageBox::information( m_pMainForm, "Hydrogen", tr("Error loading song.") );
+		return false;
+	}
+
+	return true;
 }
 
 void HydrogenApp::showMixer(bool show)
@@ -395,13 +456,13 @@ void HydrogenApp::updateWindowTitle()
 	QString title;
 
 	// special handling for initial title
-	QString qsSongName( pSong->__name );
+	QString qsSongName( pSong->getName() );
 
-	if( qsSongName == "Untitled Song" && !pSong->get_filename().isEmpty() ){
-		qsSongName = pSong->get_filename().section( '/', -1 );
+	if( qsSongName == "Untitled Song" && !pSong->getFilename().isEmpty() ){
+		qsSongName = pSong->getFilename().section( '/', -1 );
 	}
 
-	if(pSong->get_is_modified()){
+	if(pSong->getIsModified()){
 		title = qsSongName + " (" + QString(tr("modified")) + ")";
 	} else {
 		title = qsSongName;
@@ -423,6 +484,12 @@ void HydrogenApp::showAudioEngineInfoForm()
 	m_pAudioEngineInfoForm->show();
 }
 
+void HydrogenApp::showFilesystemInfoForm()
+{
+	m_pFilesystemInfoForm->hide();
+	m_pFilesystemInfoForm->show();
+}
+
 void HydrogenApp::showPlaylistDialog()
 {
 	if ( m_pPlaylistDialog->isVisible() ) {
@@ -466,10 +533,6 @@ void HydrogenApp::onDrumkitLoad( QString name ){
 	m_pPatternEditorPanel->updateSLnameLabel( );
 }
 
-void HydrogenApp::enableDestructiveRecMode(){
-	m_pPatternEditorPanel->displayorHidePrePostCB();
-}
-
 void HydrogenApp::songModifiedEvent()
 {
 	updateWindowTitle();
@@ -563,6 +626,10 @@ void HydrogenApp::onEventQueueTimer()
 				pListener->tempoChangedEvent( event.value );
 				break;
 				
+			case EVENT_UPDATE_PREFERENCES:
+				pListener->updatePreferencesEvent( event.value );
+				break;
+			
 			case EVENT_UPDATE_SONG:
 				pListener->updateSongEvent( event.value );
 				break;
@@ -571,6 +638,34 @@ void HydrogenApp::onEventQueueTimer()
 				pListener->quitEvent( event.value );
 				break;
 
+			case EVENT_TIMELINE_ACTIVATION:
+				pListener->timelineActivationEvent( event.value );
+				break;
+
+			case EVENT_TIMELINE_UPDATE:
+				pListener->timelineUpdateEvent( event.value );
+				break;
+
+			case EVENT_JACK_TRANSPORT_ACTIVATION:
+				pListener->jackTransportActivationEvent( event.value );
+				break;
+
+			case EVENT_JACK_TIMEBASE_ACTIVATION:
+				pListener->jackTimebaseActivationEvent( event.value );
+				break;
+				
+			case EVENT_SONG_MODE_ACTIVATION:
+				pListener->songModeActivationEvent( event.value );
+				break;
+				
+			case EVENT_LOOP_MODE_ACTIVATION:
+				pListener->loopModeActivationEvent( event.value );
+				break;
+
+			case EVENT_ACTION_MODE_CHANGE:
+				pListener->actionModeChangeEvent( event.value );
+				break;
+				
 			default:
 				ERRORLOG( QString("[onEventQueueTimer] Unhandled event: %1").arg( event.type ) );
 			}
@@ -579,31 +674,57 @@ void HydrogenApp::onEventQueueTimer()
 	}
 
 	// midi notes
-	while(!pQueue->m_addMidiNoteVector.empty()){
-
-		int rounds = 1;
-		if(pQueue->m_addMidiNoteVector[0].b_noteExist)// runn twice, delete old note and add new note. this let the undo stack consistent
-			rounds = 2;
-		for(int i = 0; i<rounds; i++){
-			SE_addNoteAction *action = new SE_addNoteAction( pQueue->m_addMidiNoteVector[0].m_column,
-															 pQueue->m_addMidiNoteVector[0].m_row,
-															 pQueue->m_addMidiNoteVector[0].m_pattern,
-															 pQueue->m_addMidiNoteVector[0].m_length,
-															 pQueue->m_addMidiNoteVector[0].f_velocity,
-															 pQueue->m_addMidiNoteVector[0].f_pan_L,
-															 pQueue->m_addMidiNoteVector[0].f_pan_R,
-															 0.0,
-															 pQueue->m_addMidiNoteVector[0].nk_noteKeyVal,
-															 pQueue->m_addMidiNoteVector[0].no_octaveKeyVal,
-															 false,
-															 false,
-															 pQueue->m_addMidiNoteVector[0].b_isMidi,
-															 pQueue->m_addMidiNoteVector[0].b_isInstrumentMode);
-
-			HydrogenApp::get_instance()->m_pUndoStack->push( action );
+	while( !pQueue->m_addMidiNoteVector.empty() ){
+		Song *pSong = Hydrogen::get_instance()->getSong();
+		Instrument *pInstrument = pSong->getInstrumentList()->get( pQueue->m_addMidiNoteVector[0].m_row );
+		// find if a (pitch matching) note is already present
+		Note *pOldNote = pSong->getPatternList()->get( pQueue->m_addMidiNoteVector[0].m_pattern )
+														->find_note( pQueue->m_addMidiNoteVector[0].m_column,
+																	 pQueue->m_addMidiNoteVector[0].m_column,
+																	 pInstrument,
+																	 pQueue->m_addMidiNoteVector[0].nk_noteKeyVal,
+																	 pQueue->m_addMidiNoteVector[0].no_octaveKeyVal );
+		auto pUndoStack = HydrogenApp::get_instance()->m_pUndoStack;
+		pUndoStack->beginMacro( tr( "Input Midi Note" ) );
+		if( pOldNote ) { // note found => remove it
+			SE_addOrDeleteNoteAction *action = new SE_addOrDeleteNoteAction( pOldNote->get_position(),
+																	 pOldNote->get_instrument_id(),
+																	 pQueue->m_addMidiNoteVector[0].m_pattern,
+																	 pOldNote->get_length(),
+																	 pOldNote->get_velocity(),
+																	 pOldNote->get_pan_l(),
+																	 pOldNote->get_pan_r(),
+																	 pOldNote->get_lead_lag(),
+																	 pOldNote->get_key(),
+																	 pOldNote->get_octave(),
+																	 pOldNote->get_probability(),
+																	 /*isDelete*/ true,
+																	 /*hearNote*/ false,
+																	 /*isMidi*/ false,
+																	 /*isInstrumentMode*/ false,
+																	 /*isNoteOff*/ false );
+			pUndoStack->push( action );
 		}
-		pQueue->m_addMidiNoteVector.erase(pQueue->m_addMidiNoteVector.begin());
-
+		// add the new note
+		SE_addOrDeleteNoteAction *action = new SE_addOrDeleteNoteAction( pQueue->m_addMidiNoteVector[0].m_column,
+																	 pQueue->m_addMidiNoteVector[0].m_row,
+																	 pQueue->m_addMidiNoteVector[0].m_pattern,
+																	 pQueue->m_addMidiNoteVector[0].m_length,
+																	 pQueue->m_addMidiNoteVector[0].f_velocity,
+																	 pQueue->m_addMidiNoteVector[0].f_pan_L,
+																	 pQueue->m_addMidiNoteVector[0].f_pan_R,
+																	 0.0,
+																	 pQueue->m_addMidiNoteVector[0].nk_noteKeyVal,
+																	 pQueue->m_addMidiNoteVector[0].no_octaveKeyVal,
+																	 1.0f,
+																	 /*isDelete*/ false,
+																	 false,
+																	 pQueue->m_addMidiNoteVector[0].b_isMidi,
+																	 pQueue->m_addMidiNoteVector[0].b_isInstrumentMode,
+																	 false );
+		pUndoStack->push( action );
+		pUndoStack->endMacro();
+		pQueue->m_addMidiNoteVector.erase( pQueue->m_addMidiNoteVector.begin() );
 	}
 }
 
@@ -634,27 +755,116 @@ void HydrogenApp::cleanupTemporaryFiles()
 	Filesystem::rm( Filesystem::tmp_dir(), true );
 }
 
-void HydrogenApp::updateSongEvent( int nValue ) {
+void HydrogenApp::updatePreferencesEvent( int nValue ) {
 	
-	Hydrogen* pHydrogen = Hydrogen::get_instance();	
-
+	QString sPreferencesFilename;
+	
+	// Local path of the preferences used during session management.
+	const QString sPreferencesOverwritePath = 
+		H2Core::Filesystem::getPreferencesOverwritePath();
+	if ( sPreferencesOverwritePath.isEmpty() ) {
+		sPreferencesFilename = Filesystem::usr_config_path();
+	} else {
+		sPreferencesFilename = sPreferencesOverwritePath;
+	}
+		
 	if ( nValue == 0 ) {
+		setScrollStatusBarMessage( tr("Preferences saved.") + 
+								   QString(" Into: ") + 
+								   sPreferencesFilename, 2000 );
+	} else if ( nValue == 1 ) {
+		
+		// Since the Preferences have changed, we also have to reflect
+		// these changes in the GUI - its format, colors, fonts,
+		// selections etc.
+		// But we won't change the layout!
+		Preferences *pPref = Preferences::get_instance();
+		int uiLayout = pPref->getDefaultUILayout();
+
+		WindowProperties audioEngineInfoProp = pPref->getAudioEngineInfoProperties();
+		setWindowProperties( m_pAudioEngineInfoForm, audioEngineInfoProp, SetWidth + SetHeight );
+
+		// MAINFORM
+		WindowProperties mainFormProp = pPref->getMainFormProperties();
+		setWindowProperties( m_pMainForm, mainFormProp );
+
+		m_pSplitter->setOrientation( Qt::Vertical );
+		m_pSplitter->setOpaqueResize( true );
+
+		// SONG EDITOR
+		WindowProperties songEditorProp = pPref->getSongEditorProperties();
+		setWindowProperties( m_pSongEditorPanel, songEditorProp, SetWidth + SetHeight );
+
+		// PATTERN EDITOR
+		WindowProperties patternEditorProp = pPref->getPatternEditorProperties();
+		setWindowProperties( m_pPatternEditorPanel, patternEditorProp, SetWidth + SetHeight );
+		
+		WindowProperties instrumentRackProp = pPref->getInstrumentRackProperties();
+		m_pInstrumentRack->setHidden( !instrumentRackProp.visible );
+
+		WindowProperties mixerProp = pPref->getMixerProperties();
+		if ( uiLayout != Preferences::UI_LAYOUT_SINGLE_PANE ) {
+			mixerProp.visible = false;
+		}
+		setWindowProperties( m_pMixer, mixerProp );
+
+		m_pMixer->updateMixer();
+		
+#ifdef H2CORE_HAVE_LADSPA
+		// LADSPA FX
+		for (uint nFX = 0; nFX < MAX_FX; nFX++) {
+			m_pLadspaFXProperties[nFX]->hide();
+			WindowProperties prop = pPref->getLadspaProperties(nFX);
+			setWindowProperties( m_pLadspaFXProperties[ nFX ], prop, SetX + SetY );
+		}
+#endif
+
+		// Inform the user about which file was loaded.
+		setScrollStatusBarMessage( tr("Preferences loaded.") + 
+								   QString(" From: ") + 
+								   sPreferencesFilename, 2000 );
+
+	
+	} else {
+		ERRORLOG( QString( "Unknown event parameter [%1] in HydrogenApp::updatePreferencesEvent" )
+				  .arg( nValue ) );
+	}
+	
+}
+
+void HydrogenApp::updateSongEvent( int nValue ) {
+
+	Hydrogen* pHydrogen = Hydrogen::get_instance();	
+	
+	if ( nValue == 0 || nValue == 1 ) {
 
 		// Set a Song prepared by the core part.
 		Song* pNextSong = pHydrogen->getNextSong();
+
+		if ( ! pHydrogen->getNextSongPath().isEmpty() ) {
+			pNextSong->setFilename( pHydrogen->getNextSongPath() );
+		}
+		
 		pHydrogen->setSong( pNextSong );
-	
+
 		// Cleanup
 		closeFXProperties();
 		m_pUndoStack->clear();
-	
+		
+		// Add the new loaded song in the "last used song" vector.
+		// This behavior is prohibited under session management. Only
+		// songs open during normal runs will be listed.
+		if ( ! pHydrogen->isUnderSessionManagement() ) {
+			Preferences::get_instance()->insertRecentFile( pNextSong->getFilename() );
+		}
+
 		// Update GUI components
 		m_pSongEditorPanel->updateAll();
 		m_pPatternEditorPanel->updateSLnameLabel();
 		updateWindowTitle();
 		getInstrumentRack()->getSoundLibraryPanel()->update_background_color();
 		getSongEditorPanel()->updatePositionRuler();
-		pHydrogen->getTimeline()->m_timelinetagvector.clear();
+		pHydrogen->getTimeline()->deleteAllTags();
 	
 		// Trigger a reset of the Director and MetronomeWidget.
 		EventQueue::get_instance()->push_event( EVENT_METRONOME, 2 );
@@ -664,17 +874,27 @@ void HydrogenApp::updateSongEvent( int nValue ) {
 		m_pPatternEditorPanel->updateSLnameLabel();
 		updateWindowTitle();
 		
-	} else if ( nValue == 1 ) {
+		if ( nValue == 1 ) {	
+			pHydrogen->restartDrivers();
+		}
 		
-		QString filename = pHydrogen->getSong()->get_filename();
+	} else if ( nValue == 2 ) {
+		
+		QString filename = pHydrogen->getSong()->getFilename();
 		
 		// Song was saved.
 		setScrollStatusBarMessage( tr("Song saved.") + QString(" Into: ") + filename, 2000 );
 		updateWindowTitle();
 		EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
 		
-	}
+	} else if ( nValue == 3 ) {
 
+		// The event was triggered before the Song was fully loaded by
+		// the core. It's most likely to be present by now, but it's
+		// probably better to avoid displaying its path just to be
+		// sure.
+		QMessageBox::information( m_pMainForm, "Hydrogen", tr("Song is read-only.\nUse 'Save as' to enable autosave." ) );
+	}
 }
 
 void HydrogenApp::quitEvent( int nValue ) {
