@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -32,10 +32,10 @@
 #include "../Widgets/Button.h"
 #include "../Widgets/LCD.h"
 
-#include <hydrogen/hydrogen.h>
-#include <hydrogen/Preferences.h>
-#include <hydrogen/audio_engine.h>
-#include <hydrogen/midi_action.h>
+#include <core/Hydrogen.h>
+#include <core/Preferences.h>
+#include <core/AudioEngine.h>
+#include <core/MidiAction.h>
 using namespace H2Core;
 
 #include "MixerLine.h"
@@ -80,6 +80,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 	);
 	m_pPlaySampleBtn->move( 8, 2 );
 	m_pPlaySampleBtn->setToolTip( tr( "Play sample" ) );
+	m_pPlaySampleBtn->setObjectName( "PlaySampleButton" );
 	connect(m_pPlaySampleBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));
 	connect(m_pPlaySampleBtn, SIGNAL(rightClicked(Button*)), this, SLOT(rightClick(Button*)));
 
@@ -92,6 +93,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 			QSize( 5, 13 )
 	);
 	m_pTriggerSampleLED->move( 26, 2 );
+	m_pTriggerSampleLED->setObjectName( "TriggerSampleLED" );
 	connect(m_pTriggerSampleLED, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));
 
 	// Mute button
@@ -104,6 +106,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 	);
 	m_pMuteBtn->move( 8, 17 );
 	m_pMuteBtn->setToolTip( tr( "Mute" ) );
+	m_pMuteBtn->setObjectName( "MixerMuteButton" );
 	connect(m_pMuteBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));
 
 	// Solo button
@@ -116,6 +119,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 	);
 	m_pSoloBtn->move( 30, 17);
 	m_pSoloBtn->setToolTip( tr( "Solo" ) );
+	m_pSoloBtn->setObjectName( "MixerSoloButton" );
 	connect(m_pSoloBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));
 
 	// pan rotary
@@ -159,7 +163,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 	// instrument name widget
 	m_pNameWidget = new InstrumentNameWidget( this );
 	m_pNameWidget->move( 6, 128 );
-	m_pNameWidget->setToolTip( tr( "Instrument name (double click to edit)" ) );
+	m_pNameWidget->setToolTip( tr( "Instrument name" ) );
 	connect( m_pNameWidget, SIGNAL( doubleClicked () ), this, SLOT( nameClicked() ) );
 	connect( m_pNameWidget, SIGNAL( clicked () ), this, SLOT( nameSelected() ) );
 
@@ -179,7 +183,7 @@ MixerLine::MixerLine(QWidget* parent, int nInstr)
 	m_pPeakLCD->move( 10, 106 );
 	m_pPeakLCD->setText( "0.00" );
 	QPalette lcdPalette;
-	lcdPalette.setColor( QPalette::Background, QColor( 49, 53, 61 ) );
+	lcdPalette.setColor( QPalette::Window, QColor( 49, 53, 61 ) );
 	m_pPeakLCD->setPalette( lcdPalette );
 }
 
@@ -216,11 +220,11 @@ void MixerLine::click(Button *ref) {
 	Song *song = (Hydrogen::get_instance())->getSong();
 
 	if (ref == m_pMuteBtn) {
-		song->set_is_modified(true);
+		song->setIsModified(true);
 		emit muteBtnClicked(this);
 	}
 	else if (ref == m_pSoloBtn) {
-		song->set_is_modified(true);
+		song->setIsModified(true);
 		emit soloBtnClicked(this);
 	}
 	else if (ref == m_pPlaySampleBtn) {
@@ -239,7 +243,7 @@ void MixerLine::rightClick(Button *ref)
 void MixerLine::faderChanged(Fader *ref)
 {
 	Song *pSong = (Hydrogen::get_instance())->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	emit volumeChanged(this);
 
 	double value = (double) ref->getValue();
@@ -335,24 +339,12 @@ void MixerLine::nameSelected() {
 void MixerLine::panChanged(Rotary *ref)
 {
 	Song *pSong = Hydrogen::get_instance()->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	emit panChanged( this );
-
-	float panValue = ref->getValue();
-	float pan_L, pan_R;
-	if (panValue > 0.5) {
-		pan_L = (1.0 - panValue) * 2.0;
-		pan_R = 1.0;
-	} else {
-		pan_L = 1.0;
-		pan_R = panValue * 2.0;
-	}
-
-	char m_pFaderPos[100];
-	snprintf( m_pFaderPos, 99, "%#.2fL, %#.2fR",  pan_L, pan_R);
-	HydrogenApp::get_instance()->setStatusBarMessage( tr( "Set instr. pan [%1]" ).arg( m_pFaderPos ), 2000 );
-
-	m_pPanRotary->setToolTip( QString("Pan ") + QString( m_pFaderPos ) );
+	/** Do not update tooltip nor print status message in the old fashion panL and panL style
+	 *	since inconsistent with new pan implementation. The resultant pan depends also on note pan.
+	 *	The rotary widget valuetip is enough to read the value.
+	 */
 }
 
 float MixerLine::getPan()
@@ -364,17 +356,10 @@ void MixerLine::setPan(float fValue)
 {
 	if ( fValue != m_pPanRotary->getValue() ) {
 		m_pPanRotary->setValue( fValue );
-		float pan_L, pan_R;
-		if (fValue > 0.5) {
-			pan_L = (1.0 - fValue) * 2.0;
-			pan_R = 1.0;
-		} else {
-			pan_L = 1.0;
-			pan_R = fValue * 2.0;
-		}
-		char m_pFaderPos[100];
-		snprintf( m_pFaderPos, 99,"Pan %#.2fL, %#.2fR",  pan_L, pan_R);
-		m_pPanRotary->setToolTip( QString( m_pFaderPos ) );
+		/** Do not update tooltip in the old fashion panL and panL style
+		 * since inconsistent with new pan implementation. The resultant pan depends also on note pan.
+		 * The rotary widget valuetip is enough to read the value.
+		 */
 	}
 }
 
@@ -435,7 +420,7 @@ ComponentMixerLine::ComponentMixerLine(QWidget* parent, int CompoID)
 {
 //	INFOLOG( "INIT" );
 
-	__compoID = CompoID;
+	m_nComponentID = CompoID;
 
 	m_nWidth = MIXERLINE_WIDTH;
 	m_nHeight = MIXERLINE_HEIGHT;
@@ -506,7 +491,7 @@ ComponentMixerLine::ComponentMixerLine(QWidget* parent, int CompoID)
 	m_pPeakLCD->move( 10, 106 );
 	m_pPeakLCD->setText( "0.00" );
 	QPalette lcdPalette;
-	lcdPalette.setColor( QPalette::Background, QColor( 49, 53, 61 ) );
+	lcdPalette.setColor( QPalette::Window, QColor( 49, 53, 61 ) );
 	m_pPeakLCD->setPalette( lcdPalette );
 }
 
@@ -545,11 +530,11 @@ void ComponentMixerLine::click(Button *ref) {
 	Song *pSong = (Hydrogen::get_instance())->getSong();
 
 	if (ref == m_pMuteBtn) {
-		pSong->set_is_modified( true );
+		pSong->setIsModified( true );
 		emit muteBtnClicked(this);
 	}
 	else if (ref == m_pSoloBtn) {
-		pSong->set_is_modified( true );
+		pSong->setIsModified( true );
 		emit soloBtnClicked(this);
 	}
 }
@@ -557,7 +542,7 @@ void ComponentMixerLine::click(Button *ref) {
 void ComponentMixerLine::faderChanged(Fader *ref)
 {
 	Song *pSong = (Hydrogen::get_instance())->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	emit volumeChanged(this);
 
 	double value = (double) ref->getValue();
@@ -651,13 +636,14 @@ MasterMixerLine::MasterMixerLine(QWidget* parent)
 {
 	m_nWidth = MASTERMIXERLINE_WIDTH;
 	m_nHeight = MASTERMIXERLINE_HEIGHT;
+	m_fMaxPeak = 0.0f;
 	m_nPeakTimer = 0;
 
 	setMinimumSize( m_nWidth, m_nHeight );
 	setMaximumSize( m_nWidth, m_nHeight );
 	resize( m_nWidth, m_nHeight );
 	QPalette defaultPalette;
-	defaultPalette.setColor( QPalette::Background, QColor( 58, 62, 72 ) );
+	defaultPalette.setColor( QPalette::Window, QColor( 58, 62, 72 ) );
 	this->setPalette( defaultPalette );
 
 	// Background image
@@ -682,7 +668,7 @@ MasterMixerLine::MasterMixerLine(QWidget* parent)
 	m_pPeakLCD->move( 23, 53 );
 	m_pPeakLCD->setText( "0.00" );
 	QPalette lcdPalette;
-	lcdPalette.setColor( QPalette::Background, QColor( 49, 53, 61 ) );
+	lcdPalette.setColor( QPalette::Window, QColor( 49, 53, 61 ) );
 	m_pPeakLCD->setPalette( lcdPalette );
 
 	m_pHumanizeVelocityRotary = new Rotary( this, Rotary::TYPE_NORMAL, tr( "Humanize velocity" ), false, false );
@@ -727,7 +713,7 @@ void MasterMixerLine::faderChanged(MasterFader *ref)
 	emit volumeChanged(this);
 
 	Song *pSong = Hydrogen::get_instance()->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 
 	double value = (double) ref->getValue();
 	( HydrogenApp::get_instance() )->setStatusBarMessage( tr( "Set master volume [%1]" ).arg( value, 0, 'f', 2 ), 2000 );
@@ -821,10 +807,10 @@ void MasterMixerLine::updateMixerLine()
 
 	Song *pSong = Hydrogen::get_instance()->getSong();
 	if ( pSong ) {
-		m_pHumanizeTimeRotary->setValue( pSong->get_humanize_time_value() );
-		m_pHumanizeVelocityRotary->setValue( pSong->get_humanize_velocity_value() );
-		m_pSwingRotary->setValue( pSong->get_swing_factor() );
-		m_pMuteBtn->setPressed( pSong->__is_muted );
+		m_pHumanizeTimeRotary->setValue( pSong->getHumanizeTimeValue() );
+		m_pHumanizeVelocityRotary->setValue( pSong->getHumanizeVelocityValue() );
+		m_pSwingRotary->setValue( pSong->getSwingFactor() );
+		m_pMuteBtn->setPressed( pSong->getIsMuted() );
 	}
 	else {
 		WARNINGLOG( "pSong == NULL ");
@@ -840,15 +826,15 @@ void MasterMixerLine::rotaryChanged( Rotary *pRef )
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 
 	if ( pRef == m_pHumanizeTimeRotary ) {
-		pEngine->getSong()->set_humanize_time_value( fVal );
+		pEngine->getSong()->setHumanizeTimeValue( fVal );
 		sMsg = tr( "Set humanize time parameter [%1]").arg( fVal, 0, 'f', 2 );
 	}
 	else if ( pRef == m_pHumanizeVelocityRotary ) {
-		pEngine->getSong()->set_humanize_velocity_value( fVal );
+		pEngine->getSong()->setHumanizeVelocityValue( fVal );
 		sMsg = tr( "Set humanize velocity parameter [%1]").arg( fVal, 0, 'f', 2 );
 	}
 	else if ( pRef == m_pSwingRotary ) {
-		pEngine->getSong()->set_swing_factor( fVal );
+		pEngine->getSong()->setSwingFactor( fVal );
 		sMsg = tr( "Set swing factor [%1]").arg( fVal, 0, 'f', 2 );
 	}
 	else {
@@ -916,7 +902,7 @@ void FxMixerLine::click(Button *ref) {
 	Song *pSong = Hydrogen::get_instance()->getSong();
 
 	if (ref == activeBtn ) {
-		pSong->set_is_modified( true );
+		pSong->setIsModified( true );
 		emit activeBtnClicked( this );
 	}
 }
@@ -939,7 +925,7 @@ void FxMixerLine::faderChanged(Fader *ref)
 	}
 
 	Song *pSong = Hydrogen::get_instance()->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	emit volumeChanged( this );
 }
 
@@ -1165,7 +1151,7 @@ void LadspaFXMixerLine::rotaryChanged(Rotary *ref)
 	m_fMaxPeak = 0.0;
 
 	Song *pSong = Hydrogen::get_instance()->getSong();
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	emit volumeChanged(this);
 }
 
