@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -24,9 +24,41 @@
 #define PREFERENCES_DIALOG_H
 
 
-#include "ui_PreferencesDialog_UI.h"
+#include <core/Object.h>
+#include <QtWidgets>
 
-#include <hydrogen/object.h>
+///
+/// Combo box showing a list of available devices for a given driver.
+/// List is calculated lazily when needed.
+///
+class DeviceComboBox : public QComboBox {
+
+	bool m_bHasDevices;
+	QString m_sDriver;
+	QString m_sHostAPI;
+
+public:
+	DeviceComboBox( QWidget *pParent );
+
+	/// Set the driver name to use
+	void setDriver( QString sDriver ) { m_sDriver = sDriver; }
+	void setHostAPI( QString sHostAPI ) { m_sHostAPI = sHostAPI; }
+
+	virtual void showPopup();
+};
+
+///
+/// Combo box showing a list of HostAPIs.
+///
+class HostAPIComboBox : public QComboBox {
+
+public:
+	HostAPIComboBox( QWidget *pParent );
+	void setValue( QString sHostAPI );
+	virtual void showPopup();
+};
+
+#include "ui_PreferencesDialog_UI.h"
 
 ///
 /// Preferences Dialog
@@ -36,9 +68,10 @@ class PreferencesDialog : public QDialog, private Ui_PreferencesDialog_UI, publi
 	H2_OBJECT
 	Q_OBJECT
 	public:
-		PreferencesDialog( QWidget* parent );
+		explicit PreferencesDialog( QWidget* parent );
 		~PreferencesDialog();
-
+		static QString m_sColorRed;
+							  
 	private slots:
 		void on_okBtn_clicked();
 		void on_cancelBtn_clicked();
@@ -46,6 +79,8 @@ class PreferencesDialog : public QDialog, private Ui_PreferencesDialog_UI, publi
 		void on_selectMixerFontBtn_clicked();
 		void on_restartDriverBtn_clicked();
 		void on_driverComboBox_activated( int index );
+		void on_portaudioHostAPIComboBox_activated( int index );
+		void on_latencyTargetSpinBox_valueChanged( int i );
 		void on_bufferSizeSpinBox_valueChanged( int i );
 		void on_resampleComboBox_currentIndexChanged ( int index );
 		void on_sampleRateComboBox_editTextChanged( const QString& text );
@@ -54,6 +89,7 @@ class PreferencesDialog : public QDialog, private Ui_PreferencesDialog_UI, publi
 		void on_styleComboBox_activated( int index );
 		void on_useLashCheckbox_clicked();
 		void onMidiDriverComboBoxIndexChanged( int index );
+		void on_m_pAudioDeviceTxt_currentTextChanged( QString );
 		void toggleTrackOutsCheckBox(bool toggled);
 		void toggleOscCheckBox(bool toggled);
 		void coloringMethodCombo_currentIndexChanged (int index);
@@ -64,7 +100,10 @@ class PreferencesDialog : public QDialog, private Ui_PreferencesDialog_UI, publi
 		QString m_sInitialLanguage;
 
 		void updateDriverInfo();
+
+		void updateDriverPreferences();
 };
 
+
 #endif
 
