@@ -1,6 +1,7 @@
-	/*
+/*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 #include "SongEditorPanel.h"
@@ -32,20 +32,22 @@
 #include "../Widgets/LCDCombo.h"
 #include "../Widgets/PixmapWidget.h"
 #include "../Skin.h"
+#include "../WidgetScrollArea.h"
 
 #include "SongEditor.h"
 #include "UndoActions.h"
 
-#include <hydrogen/hydrogen.h>
-#include <hydrogen/Preferences.h>
-#include <hydrogen/audio_engine.h>
-#include <hydrogen/basics/instrument_component.h>
-#include <hydrogen/basics/pattern_list.h>
+#include <core/Hydrogen.h>
+#include <core/Preferences.h>
+#include <core/AudioEngine.h>
+#include <core/Basics/InstrumentComponent.h>
+#include <core/Basics/PatternList.h>
+#include <core/IO/JackAudioDriver.h>
+
 #ifdef WIN32
 #include <time.h>
 #endif
 using namespace H2Core;
-using namespace std;
 
 const char* SongEditorPanel::__class_name = "SongEditorPanel";
 
@@ -53,7 +55,6 @@ const char* SongEditorPanel::__class_name = "SongEditorPanel";
 SongEditorPanel::SongEditorPanel(QWidget *pParent)
  : QWidget( pParent )
  , Object( __class_name )
- , m_actionMode( DRAW_ACTION )
 {
 	m_nInitialWidth = 600;
 	m_nInitialHeight = 250;
@@ -67,10 +68,12 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 
 	// background
 	PixmapWidget *pBackPanel = new PixmapWidget( nullptr );
+	pBackPanel->setObjectName( "SongEditorBackPanel" );
 	pBackPanel->setFixedSize( 196, 49 );
 	pBackPanel->setPixmap( "/songEditor/bg_topPanel.png" );
 
 	// time line toggle button
+	m_sTimelineToolTip = tr( "Enable time line edit");
 	m_pTimeLineToggleBtn = new ToggleButton(
 			pBackPanel,
 			"/songEditor/btn_bpm_on.png",
@@ -79,17 +82,18 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 			QSize( 54, 13 )
 	);
 	m_pTimeLineToggleBtn->move( 133, 6 );
-	m_pTimeLineToggleBtn->setToolTip( tr( "Enable time line edit") );
+	m_pTimeLineToggleBtn->setToolTip( m_sTimelineToolTip );
+	m_pTimeLineToggleBtn->setObjectName( "TimeLineToggleBtn" );
 	connect( m_pTimeLineToggleBtn, SIGNAL( clicked( Button* ) ), this, SLOT( timeLineBtnPressed(Button* ) ) );
 	
 	if ( pPref->getUseTimelineBpm() &&
-		 !pEngine->haveJackTimebaseClient() ) {
+		 pEngine->getJackTimebaseState() != JackAudioDriver::Timebase::Slave ) {
 		m_pTimeLineToggleBtn->setPressed( true );
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Enable time line edit") );
+		m_pTimeLineToggleBtn->setToolTip( tr( "Enable time line edit") );
 
 	} else {
 		m_pTimeLineToggleBtn->setPressed( false );
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Timeline usage is disabled in the presence of an external JACK timebase master") );
+		m_pTimeLineToggleBtn->setToolTip( tr( "Timeline usage is disabled in the presence of an external JACK timebase master") );
 
 	}
 
@@ -165,7 +169,14 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pDrawActionBtn->move( 147, 5 + 25 );
 	m_pDrawActionBtn->setToolTip( tr( "Draw mode") );
 	connect( m_pDrawActionBtn, SIGNAL( clicked( Button* ) ), this, SLOT( drawActionBtnPressed(Button* ) ) );
-	m_pDrawActionBtn->setPressed( true );
+
+	if ( pSong->getActionMode() == H2Core::Song::ActionMode::selectMode ) {
+		m_pPointerActionBtn->setPressed( true );
+		m_pDrawActionBtn->setPressed( false );
+	} else {
+		m_pPointerActionBtn->setPressed( false );
+		m_pDrawActionBtn->setPressed( true );
+	}
 
 	m_pModeActionBtn = new ToggleButton(
 			pBackPanel,
@@ -181,6 +192,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 
 // ZOOM
 	m_pHScrollBar = new QScrollBar( Qt::Horizontal, nullptr );
+	m_pHScrollBar->setObjectName( "SongEditorPanel HScrollBar" );
 	connect( m_pHScrollBar, SIGNAL(valueChanged(int)), this, SLOT( hScrollTo(int) ) );
 
 	// zoom-in btn
@@ -214,6 +226,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 			QSize( 19, 13 )
 	);
 	m_pViewPlaybackToggleBtn->setToolTip( tr( "View playback track") );
+	m_pViewPlaybackToggleBtn->setObjectName( "ViewPlaybackToggleBtn" );
 	connect( m_pViewPlaybackToggleBtn, SIGNAL( clicked( Button* ) ), this, SLOT( viewPlaybackTrackBtnPressed(Button* ) ) );
 	m_pViewPlaybackToggleBtn->setPressed( false );
 	
@@ -222,7 +235,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pPlaybackTrackFader->move( 6, 2 );
 	m_pPlaybackTrackFader->setMinValue( 0.0 );
 	m_pPlaybackTrackFader->setMaxValue( 1.5 );
-	m_pPlaybackTrackFader->setValue( pSong->get_playback_track_volume() );
+	m_pPlaybackTrackFader->setValue( pSong->getPlaybackTrackVolume() );
 	m_pPlaybackTrackFader->hide();
 	connect( m_pPlaybackTrackFader, SIGNAL( valueChanged(Fader*) ), this, SLOT( faderChanged(Fader*) ) );
 
@@ -238,7 +251,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pMutePlaybackToggleBtn->move( 151, 6 );
 	m_pMutePlaybackToggleBtn->hide();
 	connect( m_pMutePlaybackToggleBtn, SIGNAL( clicked( Button* ) ), this, SLOT( mutePlaybackTrackBtnPressed(Button* ) ) );
-	m_pMutePlaybackToggleBtn->setPressed( !pSong->get_playback_track_enabled() );
+	m_pMutePlaybackToggleBtn->setPressed( !pSong->getPlaybackTrackEnabled() );
 	
 	// edit playback track toggle button
 	m_pEditPlaybackBtn = new Button(
@@ -282,7 +295,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	//~ ZOOM
 
 	// PATTERN LIST
-	m_pPatternListScrollView = new QScrollArea( nullptr );
+	m_pPatternListScrollView = new WidgetScrollArea( nullptr );
 	m_pPatternListScrollView->setFrameShape( QFrame::NoFrame );
 	m_pPatternListScrollView->setFixedWidth( m_nPatternListWidth );
 	m_pPatternListScrollView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
@@ -290,16 +303,20 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	connect( m_pPatternListScrollView->verticalScrollBar(), SIGNAL( valueChanged(int) ), this, SLOT( vScrollTo(int) ) );
 
 	m_pPatternList = new SongEditorPatternList( m_pPatternListScrollView->viewport() );
+	m_pPatternList->setFocusPolicy( Qt::ClickFocus );
+	m_pPatternListScrollView->setFocusPolicy( Qt::ClickFocus );
 	m_pPatternListScrollView->setWidget( m_pPatternList );
 
-
 	// EDITOR
-	m_pEditorScrollView = new QScrollArea( nullptr );
+	m_pEditorScrollView = new WidgetScrollArea( nullptr );
 	m_pEditorScrollView->setFrameShape( QFrame::NoFrame );
 	m_pEditorScrollView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	m_pEditorScrollView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-	m_pSongEditor = new SongEditor( m_pEditorScrollView->viewport() );
+	m_pSongEditor = new SongEditor( m_pEditorScrollView->viewport(), m_pEditorScrollView, this );
 	m_pEditorScrollView->setWidget( m_pSongEditor );
+	m_pEditorScrollView->setFocusPolicy( Qt::ClickFocus );
+
+	m_pPatternList->setFocusProxy( m_pSongEditor );
 
 	connect( m_pEditorScrollView->horizontalScrollBar(), SIGNAL( valueChanged(int) ), this, SLOT( hScrollTo(int) ) );
 	connect( m_pEditorScrollView->verticalScrollBar(), SIGNAL( valueChanged(int) ), this, SLOT( vScrollTo(int) ) );
@@ -309,21 +326,24 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pWidgetStack = new QStackedWidget( nullptr );
 	m_pWidgetStack->setFixedHeight( 50 );
 	
-	m_pPositionRulerScrollView = new QScrollArea( m_pWidgetStack );
+	m_pPositionRulerScrollView = new WidgetScrollArea( m_pWidgetStack );
 	m_pPositionRulerScrollView->setFrameShape( QFrame::NoFrame );
 	m_pPositionRulerScrollView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	m_pPositionRulerScrollView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+	m_pPositionRulerScrollView->setFocusPolicy( Qt::NoFocus );
 	m_pPositionRuler = new SongEditorPositionRuler( m_pPositionRulerScrollView->viewport() );
 	m_pPositionRulerScrollView->setWidget( m_pPositionRuler );
 	m_pPositionRulerScrollView->setFixedHeight( 50 );
 	connect( m_pPositionRulerScrollView->horizontalScrollBar(), SIGNAL( valueChanged(int) ), this, SLOT( hScrollTo(int) ) );
+	m_pPositionRuler->setFocusPolicy( Qt::ClickFocus );
+	m_pPositionRuler->setFocusProxy( m_pSongEditor );
 	
-	m_pPlaybackTrackScrollView = new QScrollArea( m_pWidgetStack );
+	m_pPlaybackTrackScrollView = new WidgetScrollArea( m_pWidgetStack );
 	m_pPlaybackTrackScrollView->setFrameShape( QFrame::NoFrame );
 	m_pPlaybackTrackScrollView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	m_pPlaybackTrackScrollView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	
-	InstrumentComponent* pCompo = AudioEngine::get_instance()->get_sampler()->__playback_instrument->get_components()->front();
+	InstrumentComponent* pCompo = AudioEngine::get_instance()->get_sampler()->getPlaybackTrackInstrument()->get_components()->front();
 	assert(pCompo);
 
 	m_pPlaybackTrackWaveDisplay = new PlaybackTrackWaveDisplay( m_pPlaybackTrackScrollView->viewport() );
@@ -334,7 +354,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pPlaybackTrackScrollView->setWidget( m_pPlaybackTrackWaveDisplay );
 	m_pPlaybackTrackScrollView->setFixedHeight( 50 );
 	
-	m_pAutomationPathScrollView = new QScrollArea( nullptr );
+	m_pAutomationPathScrollView = new WidgetScrollArea( nullptr );
 	m_pAutomationPathScrollView->setFrameShape( QFrame::NoFrame );
 	m_pAutomationPathScrollView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	m_pAutomationPathScrollView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
@@ -352,6 +372,7 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	m_pAutomationCombo->select( 0 );
 
 	m_pVScrollBar = new QScrollBar( Qt::Vertical, nullptr );
+	m_pVScrollBar->setObjectName( "SongEditorPanel VScrollBar" );
 	connect( m_pVScrollBar, SIGNAL(valueChanged(int)), this, SLOT( vScrollTo(int) ) );
 
 	m_pWidgetStack->addWidget( m_pPositionRulerScrollView );
@@ -383,9 +404,11 @@ SongEditorPanel::SongEditorPanel(QWidget *pParent)
 	
 	this->setLayout( pGridLayout );
 	QPalette defaultPalette;
-	defaultPalette.setColor( QPalette::Background, QColor( 58, 62, 72 ) );
+	defaultPalette.setColor( QPalette::Window, QColor( 58, 62, 72 ) );
 	this->setPalette( defaultPalette );
 
+	Hydrogen::get_instance()->getSong()->setActionMode( H2Core::Song::ActionMode::selectMode );
+
 	show();
 
 	updateAll();
@@ -413,7 +436,7 @@ void SongEditorPanel::updatePlayHeadPosition()
 {
 	Song *pSong = Hydrogen::get_instance()->getSong();
 
-	if ( Preferences::get_instance()->m_bFollowPlayhead && pSong->get_mode() == Song::SONG_MODE) {
+	if ( Preferences::get_instance()->m_bFollowPlayhead && pSong->getMode() == Song::SONG_MODE) {
 		if ( Hydrogen::get_instance()->getState() != STATE_PLAYING ) {
 			return;
 		}
@@ -438,7 +461,7 @@ void SongEditorPanel::updatePlaybackFaderPeaks()
 {
 	Sampler*		pSampler = AudioEngine::get_instance()->get_sampler();
 	Preferences *	pPref = Preferences::get_instance();
-	Instrument*		pInstrument = pSampler->__playback_instrument;
+	Instrument*		pInstrument = pSampler->getPlaybackTrackInstrument();
 
 	
 	bool bShowPeaks = pPref->showInstrumentPeaks();
@@ -516,10 +539,11 @@ void SongEditorPanel::updateAll()
 
 	m_pSongEditor->createBackground();
 	m_pSongEditor->update();
-	
+
+	updatePositionRuler();
 	updateTimelineUsage();
 
- 	m_pAutomationPathView->setAutomationPath( pSong->get_velocity_automation_path() );
+ 	m_pAutomationPathView->setAutomationPath( pSong->getVelocityAutomationPath() );
 
 	resyncExternalScrollBar();
 }
@@ -527,7 +551,7 @@ void SongEditorPanel::updateAll()
 void SongEditorPanel::updatePlaybackTrackIfNecessary()
 {
 	if( Preferences::get_instance()->getShowPlaybackTrack() ) {
-		InstrumentComponent *pCompo = AudioEngine::get_instance()->get_sampler()->__playback_instrument->get_components()->front();
+		InstrumentComponent *pCompo = AudioEngine::get_instance()->get_sampler()->getPlaybackTrackInstrument()->get_components()->front();
 		m_pPlaybackTrackWaveDisplay->updateDisplay( pCompo->get_layer(0) );
 	}
 }
@@ -546,7 +570,7 @@ void SongEditorPanel::newPatBtnClicked( Button* btn )
 	UNUSED( btn );
 	Hydrogen	*pEngine = Hydrogen::get_instance();
 	Song		*pSong = pEngine->getSong();
-	PatternList *pPatternList = pSong->get_pattern_list();
+	PatternList *pPatternList = pSong->getPatternList();
 	Pattern		*pNewPattern = new Pattern( tr("Pattern %1").arg(pPatternList->size()+1));
 	PatternPropertiesDialog *pDialog = new PatternPropertiesDialog( this, pNewPattern, 0, true );
 
@@ -565,11 +589,11 @@ void SongEditorPanel::insertPattern( int idx, Pattern* pPattern )
 {
 	Hydrogen	*pEngine = Hydrogen::get_instance();
 	Song		*pSong = pEngine->getSong();
-	PatternList *pPatternList = pSong->get_pattern_list();
+	PatternList *pPatternList = pSong->getPatternList();
 
 	pPatternList->insert( idx, pPattern );
 	pEngine->setSelectedPatternNumber( idx );
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	updateAll();
 }
 
@@ -577,7 +601,7 @@ void SongEditorPanel::deletePattern( int idx )
 {
 	Hydrogen	*pEngine = Hydrogen::get_instance();
 	Song		*pSong = 	pEngine->getSong();
-	PatternList *pPatternList = pSong->get_pattern_list();
+	PatternList *pPatternList = pSong->getPatternList();
 	H2Core::Pattern *pPattern = pPatternList->get( idx );
 	
 	if( idx == 	pEngine->getSelectedPatternNumber() ) {
@@ -586,7 +610,7 @@ void SongEditorPanel::deletePattern( int idx )
 	
 	pPatternList->del( pPattern );
 	delete pPattern;
-	pSong->set_is_modified( true );
+	pSong->setIsModified( true );
 	updateAll();
 }
 
@@ -615,7 +639,7 @@ void SongEditorPanel::downBtnClicked( Button* btn )
 	UNUSED( btn );
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	Song *pSong = pEngine->getSong();
-	PatternList *pPatternList = pSong->get_pattern_list();
+	PatternList *pPatternList = pSong->getPatternList();
 
 	if( pEngine->getSelectedPatternNumber() +1 >=  pPatternList->size() ) { 
 		return;
@@ -650,7 +674,7 @@ void SongEditorPanel::clearSequence( Button* btn)
 void SongEditorPanel::restoreGroupVector( QString filename )
 {
 	//clear the old sequese
-	vector<PatternList*> *pPatternGroupsVect = Hydrogen::get_instance()->getSong()->get_pattern_group_vector();
+	std::vector<PatternList*> *pPatternGroupsVect = Hydrogen::get_instance()->getSong()->getPatternGroupVector();
 	for (uint i = 0; i < pPatternGroupsVect->size(); i++) {
 		PatternList *pPatternList = (*pPatternGroupsVect)[i];
 		pPatternList->clear();
@@ -692,37 +716,43 @@ void SongEditorPanel::resizeEvent( QResizeEvent *ev )
 	resyncExternalScrollBar();
 }
 
-void SongEditorPanel::pointerActionBtnPressed( Button* pBtn )
-{
-	pBtn->setPressed( true );
-	m_pDrawActionBtn->setPressed( false );
-	m_actionMode = SELECT_ACTION;
+void SongEditorPanel::actionModeChangeEvent( int nValue ) {
+
+	if ( nValue == 0 ) {
+		m_pPointerActionBtn->setPressed( true );
+		m_pDrawActionBtn->setPressed( false );
+	} else if ( nValue == 1 ) {
+		m_pPointerActionBtn->setPressed( false );
+		m_pDrawActionBtn->setPressed( true );
+	} else {
+		ERRORLOG( QString( "Unknown EVENT_ACTION_MODE_CHANGE value" ) );
+	}
 }
 
-
+void SongEditorPanel::pointerActionBtnPressed( Button* pBtn )
+{
+	Hydrogen::get_instance()->getSong()->setActionMode( H2Core::Song::ActionMode::selectMode );
+}
 
 void SongEditorPanel::drawActionBtnPressed( Button* pBtn )
 {
-	pBtn->setPressed( true );
-	m_pPointerActionBtn->setPressed( false );
-	m_actionMode = DRAW_ACTION;
+	Hydrogen::get_instance()->getSong()->setActionMode( H2Core::Song::ActionMode::drawMode );
 }
 
 void SongEditorPanel::updateTimelineUsage() {
 
 	auto pHydrogen = Hydrogen::get_instance();
-	
-	if ( pHydrogen->haveJackTimebaseClient() ) {
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Timeline usage is disabled in the presence of an external JACK timebase master") );
+	QString sTBMToolTip( tr( "In the presence of an external JACK Timebase master the tempo can not be altered from within Hydrogen" ) );
+	if ( pHydrogen->getJackTimebaseState() == JackAudioDriver::Timebase::Slave ) {
+		m_pTimeLineToggleBtn->setToolTip( sTBMToolTip );
 		m_pTimeLineToggleBtn->setPressed( false );
 		m_pTimeLineToggleBtn->setDisabled( true );
 		Preferences::get_instance()->setUseTimelineBpm( false );
 		m_pPositionRuler->createBackground();
 		return;
-	} else if ( m_pTimeLineToggleBtn->toolTip() == 
-				trUtf8( "Timeline usage is disabled in the presence of an external JACK timebase master") ) {
+	} else {
 		// No external timebase master present anymore.
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Enable time line edit") );
+		m_pTimeLineToggleBtn->setToolTip( m_sTimelineToolTip );
 		m_pTimeLineToggleBtn->setDisabled( false );
 	}
 	
@@ -733,11 +763,11 @@ void SongEditorPanel::timeLineBtnPressed( Button* pBtn )
 	
 	auto pHydrogen = Hydrogen::get_instance();
 	
-	if ( pHydrogen->haveJackTimebaseClient() ) {
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Timeline usage is disabled in the presence of an external JACK timebase master") );
+	if ( pHydrogen->getJackTimebaseState() == JackAudioDriver::Timebase::Slave ) {
+		m_pTimeLineToggleBtn->setToolTip( tr( "Timeline usage is disabled in the presence of an external JACK timebase master") );
 		return;
 	} else {
-		m_pTimeLineToggleBtn->setToolTip( trUtf8( "Enable time line edit") );
+		m_pTimeLineToggleBtn->setToolTip( tr( "Enable time line edit") );
 	}
 
 	if ( m_pTimeLineToggleBtn->isPressed() ){
@@ -865,12 +895,15 @@ void SongEditorPanel::setModeActionBtn( bool mode )
 void SongEditorPanel::zoomInBtnPressed( Button* pBtn )
 {
 	UNUSED( pBtn );
-	unsigned width = m_pSongEditor->getGridWidth ();
+	unsigned width = m_pSongEditor->getGridWidth();
 	--width;
-	m_pSongEditor->setGridWidth (width);
-	m_pPositionRuler->setGridWidth (width);
-	m_pAutomationPathView->setGridWidth (width);
+	m_pSongEditor->setGridWidth( width );
+	m_pPositionRuler->setGridWidth( width );
+	m_pAutomationPathView->setGridWidth( width );
 
+	Preferences::get_instance()->setSongEditorGridWidth( width );
+	Preferences::get_instance()->setSongEditorGridHeight( m_pSongEditor->getGridHeight() );
+	
 	updateAll();
 }
 
@@ -878,11 +911,15 @@ void SongEditorPanel::zoomInBtnPressed( Button* pBtn )
 void SongEditorPanel::zoomOutBtnPressed( Button* pBtn )
 {
 	UNUSED( pBtn );
-	unsigned width = m_pSongEditor->getGridWidth ();
+	unsigned width = m_pSongEditor->getGridWidth();
 	++width;
-	m_pSongEditor->setGridWidth (width);
-	m_pPositionRuler->setGridWidth (width);
-	m_pAutomationPathView->setGridWidth (width);
+	m_pSongEditor->setGridWidth( width );
+	m_pPositionRuler->setGridWidth( width );
+	m_pAutomationPathView->setGridWidth( width );
+
+	Preferences::get_instance()->setSongEditorGridWidth( width );
+	Preferences::get_instance()->setSongEditorGridHeight( m_pSongEditor->getGridHeight() );
+	
 	updateAll();
 }
 
@@ -894,7 +931,7 @@ void SongEditorPanel::faderChanged(Fader *pFader)
 	Song*		pSong = pHydrogen->getSong();
 	
 	if( pSong ){
-		pSong->set_playback_track_volume( pFader->getValue() );
+		pSong->setPlaybackTrackVolume( pFader->getValue() );
 	}
 }
 
@@ -910,7 +947,7 @@ void SongEditorPanel::automationPathChanged()
 {
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	Song *pSong = pEngine->getSong();
-	pSong->set_is_modified(true);
+	pSong->setIsModified(true);
 }
 
 
@@ -953,3 +990,15 @@ void SongEditorPanel::toggleAutomationAreaVisibility()
 	}
 }
 
+
+void SongEditorPanel::timelineActivationEvent( int nEvent ){
+	if ( nEvent == 0 && m_pTimeLineToggleBtn->isPressed() ) {
+		m_pTimeLineToggleBtn->setPressed( false );
+		HydrogenApp::get_instance()->setStatusBarMessage(tr(" Timeline = Off"), 5000);
+	} else if ( nEvent != 0 && !m_pTimeLineToggleBtn->isPressed() ) {
+		m_pTimeLineToggleBtn->setPressed( true );
+		HydrogenApp::get_instance()->setStatusBarMessage(tr(" Timeline = On"), 5000);
+	}
+	
+	m_pPositionRuler->createBackground();
+}
